@@ -1,64 +1,30 @@
 Met.PCA <-
-function(datos){
+function(datos)
+{
 
   tkconfigure(console,cursor="watch")
-
-######################################Data Load 
-  info=datos$info
-  datos<-datos$datos
-
-  dimnames(datos)[[1]]=as.character(datos[,1])
-  datos=datos[,-1] #eliminamos la primera columna
-  colnames(datos)=info$nombres
-  datos<-t(datos)
-######################################Selection of Parameters
-  valor<-((Met.Checkbox1("PCA","Scale","Center","Scale = True","Scale = False","Center = True","Center = False")))
-  escalado<-tclvalue(valor[1])
-  if (escalado == "1") 
-  {
-escalado=TRUE
-  }
-  else 
-escalado=FALSE
-  centro<-tclvalue(valor[2])
-  if (centro == "1") 
-  centro=TRUE
-  else 
-centro=FALSE
-
-######################################PCA
-  Try(prcomp(datos,scale=escalado,
-center=centro,tol = sqrt(.Machine$double.eps)))
-  dat.pc<-prcomp(datos,scale=escalado,
-center=centro,tol = sqrt(.Machine$double.eps))
-  varianza<-summary(dat.pc,loadings=TRUE,digits=2)
-  Principales<-predict(dat.pc)
-  varianza<-round(varianza$importance[2,],4)*100 
-  ncomp<-length(varianza)
+  Require("robustbase")
+  ############################### Score Plot
   
-######################################2D PLOT
-  f2<-function(){
-  etiqueta=info[,1]
-  color="white"
-  color2="black"
-  col=list(axis="black",lab="black",main="black",sub="black")
-  main<-" "
- subt<-" "  
-  size=list(cex=0.7,cex.axis=1,cex.lab=1,cex.main=1,cex.sub=1)
-  xlab<-paste("PC1 (",varianza[1],"%)")
-  ylab<-paste("PC2 (",varianza[2],"%)")
-  PLSC=list(a=1,b=2)
+  f2<-function(a,etiqueta=info[,1],color="white",color2="black",col=list(axis="red",lab="blue",main="black",sub="black"),
+  main="Tolerance ellipse (97.5%)",subt=" ",size=list(cex=0.7,cex.axis=1,cex.lab=1,cex.main=1,cex.sub=1),
+  xlab=paste("PC1 (",varianza[1],"%)"), ylab=paste("PC2 (",varianza[2],"%)"),PLSC=list(a=1,b=2))
+  {
+
 tt <- tktoplevel()
 plotFunction <- function(color,color2,etiqueta,xlab,ylab,main,subt)
 {
   par(bg=color)
-plot(Principales[,PLSC$a],Principales[,PLSC$b],
-ylim=range(Principales[,PLSC$b]),xlim=range(Principales[,PLSC$a]),
-xlab=xlab,ylab=ylab,main=main,sub=subt,type="n",cex.axis=size$cex.axis,
-cex.lab=size$cex.lab,cex.main=size$cex.main,cex.sub=size$cex.sub,
-col.axis=col$axis,col.lab=col$lab,col.main=col$main,col.sub=col$sub)
-text(Principales[,PLSC$a],Principales[,PLSC$b],
-labels=abbreviate(etiqueta),cex=size$cex,col=color2)
+tolEllipsePlot(a[,c(PLSC$a,PLSC$b)], m.cov = covMcd(a[,c(PLSC$a,PLSC$b)]), 
+   cutoff = NULL, id.n = 0,
+               classic = TRUE, tol = 1e-07,
+               xlab = xlab, ylab = ylab,
+               main =main, txt.leg = c("robust", "classical"),
+               col.leg = c(col$axis, col$lab),
+               lty.leg = c("solid","dashed"))
+text(a[,PLSC$a],a[,PLSC$b],cex.lab=size$cex.lab,pos=1,
+labels=abbreviate(etiqueta),cex=size$cex,col=color2,
+)
 }
 img <- tkrplot(tt,plotFunction(color,color2,etiqueta,xlab,ylab,main,subt),hscale=1.5,vscale=1.5)
 
@@ -92,18 +58,7 @@ ChangeColor4 <- function()
      if (nchar(col$lab)>0)
         tkconfigure(canvas4,bg=col$lab)
      }
-ChangeColor5 <- function()
-    {
-     col$main <<- tclvalue(.Tcl(paste("tk_chooseColor",.Tcl.args(initialcolor=col$main,title="Choose a color"))))
-     if (nchar(col$main)>0)
-        tkconfigure(canvas5,bg=col$main)
-     }
-ChangeColor6 <- function()
-    {
-     col$sub <<- tclvalue(.Tcl(paste("tk_chooseColor",.Tcl.args(initialcolor=col$sub,title="Choose a color"))))
-     if (nchar(col$sub)>0)
-        tkconfigure(canvas6,bg=col$sub)
-     }
+
   canvas1 <- tkcanvas(tt,width="80",height="25",bg=color)
     canvas2 <- tkcanvas(tt,width="80",height="25",bg=color2)
   canvas3 <- tkcanvas(tt,width="80",height="25",bg=col$axis)
@@ -114,8 +69,6 @@ canvas6 <- tkcanvas(tt,width="80",height="25",bg=col$sub)
     ChangeColor.button2 <- tkbutton(tt,text="Change Color",command=function() ChangeColor2())
 ChangeColor.button3 <- tkbutton(tt,text="Change Color",command=function() ChangeColor3())
 ChangeColor.button4 <- tkbutton(tt,text="Change Color",command=function() ChangeColor4())
-ChangeColor.button5 <- tkbutton(tt,text="Change Color",command=function() ChangeColor5())
-ChangeColor.button6 <- tkbutton(tt,text="Change Color",command=function() ChangeColor6())
 onOK <- function()
   {
     tkrreplot(img,plotFunction(color,color2,etiqueta,xlab,ylab,main,subt))
@@ -133,14 +86,11 @@ onOK <- function()
 tkgrid(tklabel(tt,text="    "))
 tkgrid(tklabel(tt,text="Label    "),canvas2,ChangeColor.button2)
    tkgrid(tklabel(tt,text="    "))
-tkgrid(tklabel(tt,text="Axis    "),canvas3,ChangeColor.button3)
+tkgrid(tklabel(tt,text="Robust    "),canvas3,ChangeColor.button3)
    tkgrid(tklabel(tt,text="    "))
-tkgrid(tklabel(tt,text="Legend    "),canvas4,ChangeColor.button4)
-   tkgrid(tklabel(tt,text="    "))
-tkgrid(tklabel(tt,text="Title    "),canvas5,ChangeColor.button5)
-   tkgrid(tklabel(tt,text="    "))
-tkgrid(tklabel(tt,text="Subtitle    "),canvas6,ChangeColor.button6)
+tkgrid(tklabel(tt,text="Classical    "),canvas4,ChangeColor.button4)
    tkgrid(tklabel(tt,text="    "))
+
 tkgrid(OK.but,Cancel.but)
    }
 
@@ -217,41 +167,20 @@ dlg <- tktoplevel()
   tkfocus(dlg)
   tkwm.title(dlg,"Text Size")
 SliderValue1 <- tclVar("0.7")
-SliderValue2 <- tclVar("1")
-SliderValue3 <- tclVar("1")
-SliderValue4 <- tclVar("2")
-SliderValue5 <- tclVar("1")
+
 SliderValueLabel1 <- tklabel(dlg,text=as.character(tclvalue(SliderValue1)))
-SliderValueLabel2 <- tklabel(dlg,text=as.character(tclvalue(SliderValue2)))
-SliderValueLabel3 <- tklabel(dlg,text=as.character(tclvalue(SliderValue3)))
-SliderValueLabel4 <- tklabel(dlg,text=as.character(tclvalue(SliderValue4)))
-SliderValueLabel5 <- tklabel(dlg,text=as.character(tclvalue(SliderValue5)))
 tkconfigure(SliderValueLabel1,textvariable=SliderValue1)
-tkconfigure(SliderValueLabel2,textvariable=SliderValue2)
-tkconfigure(SliderValueLabel3,textvariable=SliderValue3)
-tkconfigure(SliderValueLabel4,textvariable=SliderValue4)
-tkconfigure(SliderValueLabel5,textvariable=SliderValue5)
 
 
 slider1 <- tkscale(dlg, from=4, to=0,showvalue=F, variable=SliderValue1,
                    resolution=0.1, orient="horizontal")
-slider2 <- tkscale(dlg, from=4, to=0,showvalue=F, variable=SliderValue2,
-                   resolution=0.1, orient="horizontal")
-slider3 <- tkscale(dlg, from=4, to=0,showvalue=F, variable=SliderValue3,
-                   resolution=0.1, orient="horizontal")
-slider4 <- tkscale(dlg, from=4, to=0,showvalue=F, variable=SliderValue4,
-                   resolution=0.1, orient="horizontal")
-slider5 <- tkscale(dlg, from=4, to=0,showvalue=F, variable=SliderValue5,
-                   resolution=0.1, orient="horizontal")
+
 tkgrid(tklabel(dlg,text="       "))
 tkgrid(tklabel(dlg,text="Text Size"),sticky="w")
 tkgrid(tklabel(dlg,text="       "))
 
 tkgrid(tklabel(dlg,text="Labels : "),SliderValueLabel1,slider1)
-tkgrid(tklabel(dlg,text="Axis : "),SliderValueLabel2,slider2)
-tkgrid(tklabel(dlg,text="Legend : "),SliderValueLabel3,slider3)
-tkgrid(tklabel(dlg,text="Title : "),SliderValueLabel4,slider4)
-tkgrid(tklabel(dlg,text="Subtitle : "),SliderValueLabel5,slider5)
+
 
 tkgrid(tklabel(dlg,text="       "))
   tkgrid(tklabel(dlg,text="       "))
@@ -259,9 +188,7 @@ tkgrid(tklabel(dlg,text="       "))
 onOK <- function()
   {
 size<<-list(
-    cex=as.numeric(tclvalue(SliderValue1)),cex.axis=as.numeric(tclvalue(SliderValue2)),
-    cex.lab=as.numeric(tclvalue(SliderValue3)),cex.main=as.numeric(tclvalue(SliderValue4)),
-cex.sub=as.numeric(tclvalue(SliderValue5)))
+    cex=as.numeric(tclvalue(SliderValue1)))
     tkdestroy(dlg)
 tkrreplot(img,plotFunction(color,color2,etiqueta,xlab,ylab,main,subt))  
    }
@@ -375,21 +302,15 @@ tkfocus(tt)
 tkraise(tt)
 #tkwait.window(tt)
 }
-  f2()
 
-########################################################Loadings
 
-  f2<-function(){
-  main<-"Loading"
-  xlab<-paste("PC1 (",varianza[1],"%)")
-  ylab<-paste("PC2 (",varianza[2],"%)")
-  PLSC=list(a=1,b=2) 
-  etiqueta=info[,1]
-  color="white"
-  color2="black"
-  col=list(axis="black",lab="black",main="black",sub="black")
-  subt=""
-  size=list(cex=0.7,cex.axis=1,cex.lab=1,cex.main=1,cex.sub=1)
+  #####################################Loadings Plot
+
+  f2b<-function(main="Loading",xlab=paste("PC1 (",varianza[1],"%)"),ylab=paste("PC2 (",varianza[2],"%)"),
+      PLSC=list(a=1,b=2),etiqueta=info[,1],color="white",color2="black",col=list(axis="black",lab="black",main="black",sub="black"),
+      subt="",size=list(cex=0.7,cex.axis=1,cex.lab=1,cex.main=1,cex.sub=1))
+  {
+  
 tt <- tktoplevel()
 plotFunction <- function(color,color2,etiqueta,xlab,ylab,main,subt)
 {
@@ -700,7 +621,281 @@ tkraise(tt)
 #tkwait.window(tt)
 
   }
-  f2()
+  ############################### Model Information
+
+  f2c<-function(a,w,etiqueta=info[,1],color="white",color2="black",col=list(axis="red",lab="blue",main="black",sub="black"),
+  main="Tolerance ellipse (97.5%)",subt=" ",size=list(cex=0.7,cex.axis=1,cex.lab=1,cex.main=1,cex.sub=1),
+  xlab=paste("PC1 (",varianza[1],"%)"), ylab=paste("PC2 (",varianza[2],"%)"),PLSC=list(a=1,b=2))
+  {
+
+tt <- tktoplevel()
+plsplottype <- c("dd", "distance", "qqchi2", "screeplot","standard deviations of the principal components")
+
+plotFunction <- function()
+{
+if (w<5)
+{
+  par(bg=color)
+covPlot(Principales[,c(PLSC$a,PLSC$b)],which =plsplottype[w],
+classic = TRUE,
+     labels.id = abbreviate(etiqueta), cex.id = size$cex,
+     label.pos = c(4,2), tol = 1e-7)
+}
+if (w==5)
+{
+par(bg=color)
+screeplot(dat.pc, npcs = min(10, length(dat.pc$sdev)),
+          type = c("lines"),
+          main ="Standard deviations of the principal components")
+}
+
+}
+img <- tkrplot(tt,plotFunction(),hscale=2.5,vscale=1.5)
+
+
+
+by.name<-function(info)
+{
+    etiqueta<<-info[,1]
+tkrreplot(img,plotFunction())
+  }
+
+by.type<-function(info)
+{
+   etiqueta<<-info[,categoria] 
+tkrreplot(img,plotFunction())
+}
+
+Text.size<- function()
+{
+dlg <- tktoplevel()
+  tkwm.deiconify(dlg)
+  tkgrab.set(dlg)
+  tkfocus(dlg)
+  tkwm.title(dlg,"Text Size")
+SliderValue1 <- tclVar("0.7")
+
+SliderValueLabel1 <- tklabel(dlg,text=as.character(tclvalue(SliderValue1)))
+tkconfigure(SliderValueLabel1,textvariable=SliderValue1)
+
+
+slider1 <- tkscale(dlg, from=4, to=0,showvalue=F, variable=SliderValue1,
+                   resolution=0.1, orient="horizontal")
+
+tkgrid(tklabel(dlg,text="       "))
+tkgrid(tklabel(dlg,text="Text Size"),sticky="w")
+tkgrid(tklabel(dlg,text="       "))
+
+tkgrid(tklabel(dlg,text="Labels : "),SliderValueLabel1,slider1)
+
+
+tkgrid(tklabel(dlg,text="       "))
+  tkgrid(tklabel(dlg,text="       "))
+  
+onOK <- function()
+  {
+size<<-list(
+    cex=as.numeric(tclvalue(SliderValue1)))
+    tkdestroy(dlg)
+tkrreplot(img,plotFunction())  
+   }
+  onCancel <- function()
+  {
+    ReturnVal <<- 0
+    tkdestroy(dlg)
+   }
+  OK.but     <-tkbutton(dlg,text="   OK   ",command=onOK)
+  Cancel.but <-tkbutton(dlg,text=" Cancel ",command=onCancel)
+  tkgrid(OK.but,Cancel.but)
+  tkgrid(tklabel(dlg,text="    "))
+  tkraise(dlg)
+  tkfocus(dlg)
+  tkbind(dlg, "<Destroy>", function() {tkgrab.release(dlg);tkfocus(dlg)})
+  tkwait.window(dlg)
+
+  return(ReturnVal)
+
+  }
+
+
+CopyToClip <- function()
+{
+fileName<-tclvalue(tkgetSaveFile())
+pdf(file = fileName)
+plotFunction()
+dev.off()
+#windows()
+  }
+onok <- function()
+{
+tkdestroy(tt)}
+Comp <- function()
+{
+dlg <- tktoplevel()
+  tkwm.deiconify(dlg)
+  tkgrab.set(dlg)
+  tkfocus(dlg)
+  tkwm.title(dlg,"PCA Components")
+SliderValue1 <- tclVar(PLSC$a)
+SliderValue2 <- tclVar(PLSC$b)
+SliderValueLabel1 <- tklabel(dlg,text=as.character(tclvalue(SliderValue1)))
+SliderValueLabel2 <- tklabel(dlg,text=as.character(tclvalue(SliderValue2)))
+tkconfigure(SliderValueLabel1,textvariable=SliderValue1)
+tkconfigure(SliderValueLabel2,textvariable=SliderValue2)
+slider1 <- tkscale(dlg, from=1, to=ncomp,showvalue=F, variable=SliderValue1,
+                   resolution=1, orient="horizontal")
+slider2 <- tkscale(dlg, from=1, to=ncomp,showvalue=F, variable=SliderValue2,
+                   resolution=1, orient="horizontal")
+tkgrid(tklabel(dlg,text="       "))
+tkgrid(tklabel(dlg,text="X axis : "),SliderValueLabel1,slider1)
+tkgrid(tklabel(dlg,text="       "))
+tkgrid(tklabel(dlg,text="Y axis : "),SliderValueLabel2,slider2)
+tkgrid(tklabel(dlg,text="       "))
+  tkgrid(tklabel(dlg,text="       "))
+  
+onOK <- function()
+  {
+PLSC<<-list(a=as.numeric(tclvalue(SliderValue1)),b=as.numeric(tclvalue(SliderValue2))) 
+  xlab<-paste("PC",PLSC$a,"(",varianza[PLSC$a],"%)")
+  ylab<-paste("PC",PLSC$b,"(",varianza[PLSC$b],"%)")
+tkdestroy(dlg)
+tkrreplot(img,plotFunction())  
+   }
+
+  onCancel <- function()
+  {
+    ReturnVal <<- 0
+    tkdestroy(dlg)
+   }
+
+  OK.but     <-tkbutton(dlg,text="   OK   ",command=onOK)
+  Cancel.but <-tkbutton(dlg,text=" Cancel ",command=onCancel)
+  tkgrid(OK.but,Cancel.but)
+  tkgrid(tklabel(dlg,text="    "))
+  tkraise(dlg)
+  tkfocus(dlg)
+  tkbind(dlg, "<Destroy>", function() {tkgrab.release(dlg);tkfocus(dlg)})
+  tkwait.window(dlg)
+
+  }
+pls.mode<-function()
+ {
+entryWidth=7
+  tt <- tktoplevel()
+  fontHeading <- tkfont.create(family="times",size=12,weight="bold")
+  fontTextLabel <- tkfont.create(family="times",size=12)
+  tkwm.deiconify(tt)
+  tkgrab.set(tt)
+  tkfocus(tt)
+  tkwm.title(tt,"PCA")
+  tkgrid(tklabel(tt,text="    "))
+  tkgrid(tklabel(tt,text="Mode :",font=fontHeading),sticky="e")
+  tkgrid(tklabel(tt,text="    "))
+rbuton<-list()
+for (i in 1:length(plsplottype))
+rbuton[[i]]<-tkradiobutton(tt)
+
+  rbValue<-tclVar(w)
+for (i in 1:length(plsplottype))
+  tkconfigure(rbuton[[i]],variable=rbValue,value=i)
+  
+for (i in 1:length(plsplottype))
+tkgrid(tklabel(tt,text=plsplottype [i]),rbuton[[i]],sticky="e")
+
+   tkgrid(tklabel(tt,text="       "))
+    onOK <- function()
+  {
+    w<<- as.numeric((tclvalue(rbValue)))
+    tkgrab.release(tt)
+    tkdestroy(tt)
+tkrreplot(img,plotFunction())  
+
+     }
+  onCancel <- function()
+  {
+    categoria <<- 0
+    tkdestroy(tt)
+   }
+  OK.but     <-tkbutton(tt,text="   OK   ",command=onOK)
+  Cancel.but <-tkbutton(tt,text=" Cancel ",command=onCancel)
+    tkgrid(tklabel(tt,text="    "))
+  tkgrid(tklabel(tt,text="    "),OK.but,Cancel.but,tklabel(tt,text="    "),sticky="w")
+  tkgrid(tklabel(tt,text="    "))
+  tkfocus(tt)
+  tkraise(tt)
+  tkwait.window(tt)
+  }
+
+
+tkwm.title(tt,"Interactive Plot")
+Menu <- tkmenu(tt,borderwidth=40)
+tkconfigure(tt, menu=Menu)
+labels <- tkmenu(Menu,borderwidth=40,tearoff=FALSE)
+tkadd(labels, "command", label="Name",
+      command=function() by.name(info))
+tkadd(labels, "command", label="Type",
+      command=function() by.type(info))
+tkadd(Menu, "cascade", label="Labels",menu=labels)
+Legend <- tkmenu(Menu,borderwidth=40,tearoff=FALSE)
+tkadd(Legend, "command", label="Size",
+      command=function() Text.size())
+tkadd(Menu, "cascade", label="Legend",menu=Legend)
+tkadd(Menu, "command", label="Components",
+      command=function() Comp())
+tkadd(Menu, "command", label="Mode",
+      command=function() pls.mode())
+
+copy.but <- tkbutton(tt,text="Copy to pdf",command=CopyToClip)
+ok.but <- tkbutton(tt,text="Ok",command=onok)
+tkpack(img,side="top")
+tkpack(ok.but,side="right",padx="120")
+tkpack(copy.but,side="left",padx="120")
+tkfocus(tt)
+tkraise(tt)
+#tkwait.window(tt)
+}
+
+  ######################################PCA Analysis
+  info=datos$info
+  datos<-datos$datos
+  dimnames(datos)[[1]]=as.character(datos[,1])
+  datos=datos[,-1] #eliminamos la primera columna
+  colnames(datos)=info$nombres
+  datos<-t(datos)
+
+  valor<-((Met.Checkbox1("PCA","Scale","Center",
+"Scale = True","Scale = False","Center = True","Center = False"))) #Selection of Parameters
+  escalado<-tclvalue(valor[1])
+  if (escalado == "1") 
+  {
+escalado=TRUE
+  }
+  else 
+escalado=FALSE
+  centro<-tclvalue(valor[2])
+  if (centro == "1") 
+  centro=TRUE
+  else 
+centro=FALSE
+
+
+  dat.pc<-Try(prcomp(datos,scale=escalado,
+center=centro,tol = sqrt(.Machine$double.eps)))
+  varianza<-summary(dat.pc,loadings=TRUE,digits=2)
+  Principales<-predict(dat.pc)
+  varianza<-round(varianza$importance[2,],4)*100 
+  ncomp<-length(varianza)
+  
+  f2(Principales)
+  f2b()
+  f2c(Principales,w=1)
+ 
+  Standar.Deviations<-as.data.frame(dat.pc$sdev)
+  colnames(Standar.Deviations)<-"Standar Deviations"
+  rownames(Standar.Deviations)<- colnames(dat.pc$rotation)
+  showData2(cbind(info,Principales), title="PC Scores")
+  showData2(dat.pc$rotation,title="PC Loadings")
+  showData2(Standar.Deviations,title="PCA Standard deviations of the principal components")
   tkconfigure(console,cursor="arrow")
 }
 
